--[[

This file is part of RampageAI.

RampageAI is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

RampageAI is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with RampageAI; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

--]]

--------------------
-- Initialization --
--------------------

function InitPersonality()
	-- Set the acknowledgement functions
	Personality_Yes = AI_PersoYes
	Personality_No = AI_PersoNo
	Personality_Other = AI_PersoOther

	-- Plot 5 circles (radius 2-6)
	local angleIncs = { 45, 30, 20, 15, 10 }
	local i
	for i=2,6 do
		-- Make a new table for this radius
		PersoActions.circle[i] = Table.new()

		-- Give it a name
		PersoActions.circle[i].name = "circle[" .. i .. "]"

		-- Setup to make the coords for the table
		local t,inc,angle = PersoActions.circle[i],angleIncs[i-1],0
		local lastOpt = math.floor(i / 2)
		local lastX,lastY

		-- Make the coordinates for the table
		while angle < 360 do
			-- Plot the coordinates around (0,0)
			local x,y = PPlotCircle(0,0,angle,i)

			-- Make sure its not too close to the old one
			if lastX == nil or PPDistance(lastX,lastY,x,y) > math.floor(i / 3) then
				-- Insert into the table
				Table.insert(t, { x, y, lastOpt } )

				-- Set the lastX and lastY
				lastX,lastY = x,y
			end

			-- Increment the angle
			angle = angle + inc
		end
	end

	-- Set the idle movement function
	if not EnableIdleMove then
		Personality_Idle = NoReturn
	else
		-- Set the function
		Personality_Idle = AI_Idle
	end
end

------------------
-- Yes/No/Other --
------------------

function AI_PersoYes(target,rots)
	-- Check if we have a number of rotations
	if type(rots) ~= "number" then
		-- We don't, use default
		rots = 2
	end

	-- Add a circle command to the sequencer
	List.pushright(AI_seq, { SEQ_PREROUTED, target, PersoActions.circle[3], rots, true })
end

function AI_PersoNo(target,rots)
	-- Check if we have a number of rotations
	if type(rots) ~= "number" then
		-- Use default
		rots = 1
	end

	-- Add a zigzag command to the sequencer
	List.pushright(AI_seq, { SEQ_PREROUTED, target, PersoActions.zigzag, rots })
end

function AI_PersoOther(target,movement,rots)
	-- Check if we have a number of rotations
	if type(rots) ~= "number" then
		-- Use default
		rots = 2
	end

	-- Add a prerouted command to the sequencer
	List.pushright(AI_seq, { SEQ_PREROUTED, target, movement, rots })
end

----------
-- Idle --
----------

function AI_Idle()
	-- Check if we're holding position
	if HoldPosition then
		return
	end

	-- Check if the alchemist has moved recently
	if OwnerLast.Move + 1000 > GetTick() then
		return
	end

	-- Check if we have enough HP/SP
	if ActorHP(HomunID) >= IdleMoveHP and
		ActorSP(HomunID) >= IdleMoveSP then

		-- Check if we're on the same idle sequence
		if not PersoIdleAction or
			PersoIdleAction.lastSeq ~= HomunLast.Sequence then

			-- Create a sequence
			local seq = { SEQ_PREROUTED, OwnerID, PersoActions.circle[IdleMoveRadius], 1, false,
					loglevel=6, allowBypass=true }
	
			--[[ Disabled so as to not confuse with Personality_Yes()
	
			-- Decide if we want to go clockwise or not
			if math.random(1,2) == 2 then
				-- Set clockwise
				seq[5] = true
			end
			--]]
	
			-- Make a list of possible people to circle around
			local actor,dummy
			local t = Table.new()
			Table.insert(t,OwnerID)		-- Add alchemist as an option
			for actor,dummy in pairs(PermFriendsOnScr) do
				-- Make sure its true
				if dummy then
					Table.insert(t,actor)
				end
			end
	
			-- Decide on the person to circle around
			seq[2] = t[math.random(1,Table.getn(t))]
	
			-- Check if the target is a friend
			if seq[2] ~= OwnerID then
				-- Make it a smaller circle
				seq[3] = PersoActions.circle[3]
			end
	
			-- Add to the sequencer (disabled so it won't reset the HomunLast.Sequence)
			-- List.pushright(AI_seq,seq)
			seq.lastSeq = HomunLast.Sequence
			PersoIdleAction = seq
		end

		-- Call the prerouted function
		local tryagain,seq = true,PersoIdleAction
		while tryagain and seq do
			tryagain,seq = SEQ_PreRouted(seq)
		end
		PersoIdleAction = seq

	else
		-- Check how close they are to us
		if AABlocks(HomunID,OwnerID) > HomunFollowDistance then
			-- Make the homunculus come close to us to regen
			List.pushright(AI_seq,{ SEQ_MOVE_TO, OwnerID, HomunFollowDistance })

			-- Don't do anything else
			return
		end
	end
end

-------------
-- Actions --
-------------

PersoActions = {

	-- circle begin
	circle = {
		-- this one has multiple radiuses
		multiradius = true,

		-- No radius of 1
		false,

		-- the rest are auto-generated

	},
	-- circle end

	-- heart begin
	heart = {	name="heart",

			{  0,  1 },	-- "top"

			{ -1,  2 },	-- left side
			{ -2,  3 },
			{ -3,  1 },
			{ -2, -1 },

			{  0, -3 },	-- "bottom"

			{  2, -1 },	-- right side
			{  3,  1 },
			{  2,  3 },
			{  1,  2 },
	},
	-- heart end

	-- zigzag begin
	zigzag = {	name="zigzag",

			{ -2,  2 },
			{  2,  2 },
			{ -2,  0 },
			{  2,  0 },
			{ -2, -2 },
			{  2, -2 },

			-- Reverse directions when this step is hit
			{ bounce=true },
	},
	-- zigzag end
}
